// Fragment shader for per-pixel Phong interpolation and shading.

// The "varying" keyword means that the parameter's value is interpolated
// between the nearby vertices.
varying vec3 N;
varying vec3 v;

uniform float brightness;

void main()
{
	vec4 ambient, diffuse, specular;
	vec3 Normal, Viewer;
	vec3 DirectionalLight, DirectionalHalf;
	vec3 PointSourceVector, PointLight, PointHalf;
	float DDiffuseShade, DSpecularShade;
	float PDiffuseShade, PSpecularShade, pointDistance;

	// The scene's ambient light.
	ambient = gl_LightModel.ambient * gl_FrontMaterial.ambient;

	// The normal vectors is generally not normalized after being
	// interpolated across a triangle.  Here we normalize it.
	Normal = normalize(N);

	// Since the vertex is in eye space, the direction to the
	// viewer is simply the normalized vector from v to the
	// origin.
	Viewer = -normalize(v);

	// TODO: optional, reduce redundancy
	
	//// Directional Component
	// Get the lighting direction and normalize it.
	DirectionalLight = normalize(gl_LightSource[0].position.xyz);

	// Compute halfway vector
	DirectionalHalf = normalize(Viewer + DirectionalLight);

	// Compute factor to prevent light leakage from below the
	// surface
	float B = 1.0;
	if (dot(Normal, DirectionalLight)<0.0) B = 0.0;

	// Compute geometric terms of diffuse and specular
	DDiffuseShade = max(dot(Normal, DirectionalLight), 0.0);
	DSpecularShade =
		B * pow(max(dot(DirectionalHalf, Normal), 0.0), gl_FrontMaterial.shininess);

	// Compute product of geometric terms with material and
	// lighting values
	diffuse = DDiffuseShade * gl_FrontLightProduct[0].diffuse;
	specular = DSpecularShade * gl_FrontLightProduct[0].specular;
	ambient += gl_FrontLightProduct[0].ambient;

	//// Point Component
	// Get the difference vector between current fragment and the source
	PointSourceVector = gl_LightSource[1].position.xyz - v;

	// Normalize the lighting vector
	PointLight = normalize(PointSourceVector);

	// Compute halfway vector
	PointHalf = normalize(Viewer + PointLight);
	pointDistance = length(PointSourceVector);

	// Compute attenuation due to distance from source
	float attenuation = 1.0 /
		(gl_LightSource[1].quadraticAttenuation	* pow(pointDistance, 2)
	   + gl_LightSource[1].linearAttenuation	* pointDistance
	   + gl_LightSource[1].constantAttenuation);

	// Compute factor to prevent light leakage from below the 
	// surface
	B = 1.0;
	if (dot(Normal, PointLight)<0.0) B = 0.0;
	
	// Compute geometric terms of diffuse and specular
	PDiffuseShade = max(dot(Normal, PointLight), 0.0);
	PSpecularShade = B * pow(max(dot(PointHalf, Normal), 0.0), gl_FrontMaterial.shininess);

	// Compute product of geometric terms with material and 
	// lighting values attenuated by distance
	diffuse += attenuation * (PDiffuseShade * gl_FrontLightProduct[1].diffuse);
	specular += attenuation * (PSpecularShade * gl_FrontLightProduct[1].specular);
	ambient += attenuation * gl_FrontLightProduct[1].ambient;

	// Assign final color
	gl_FragColor = ambient + diffuse + specular + gl_FrontMaterial.emission;
	// Scale with brightness value
	gl_FragColor = gl_FragColor * brightness;
}